"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
Object.defineProperty(exports, "cache", {
    enumerable: true,
    get: ()=>cache
});
const _fs = /*#__PURE__*/ _interopRequireDefault(require("fs"));
const _path = /*#__PURE__*/ _interopRequireDefault(require("path"));
const _logger = require("../logger");
const _getWorkspace = require("../workspace/getWorkspace");
function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        default: obj
    };
}
const MS_IN_A_DAY = 1000 * 60 * 60 * 24;
async function cache(cwd, config) {
    if (config.clear) {
        _logger.logger.info("clearing cache, this may take a while");
        clearCache(cwd, config);
        _logger.logger.info("done!");
    } else if (config.prune) {
        _logger.logger.info("pruning cache, this may take a while");
        pruneCache(cwd, config);
        _logger.logger.info("done!");
    } else {
        _logger.logger.info("No options given to cache command. Try --clear or --prune");
    }
}
function clearCache(cwd, config) {
    const workspace = (0, _getWorkspace.getWorkspace)(cwd, config);
    const { allPackages  } = workspace;
    for (const info of Object.values(allPackages)){
        const cachePath = getCachePath(info, config.cacheOptions);
        if (_fs.default.existsSync(cachePath)) {
            const entries = _fs.default.readdirSync(cachePath);
            for (const entry of entries){
                _logger.logger.verbose(`clearing cache for ${info.name}`);
                const entryPath = _path.default.join(cachePath, entry);
                const entryStat = _fs.default.statSync(entryPath);
                remove(entryPath, entryStat);
            }
        }
    }
}
function pruneCache(cwd, config) {
    const prunePeriod = parseInt(config.prune) || 30;
    const now = new Date();
    const workspace = (0, _getWorkspace.getWorkspace)(cwd, config);
    const { allPackages  } = workspace;
    for (const info of Object.values(allPackages)){
        const cachePath = getCachePath(info, config.cacheOptions);
        if (_fs.default.existsSync(cachePath)) {
            const entries = _fs.default.readdirSync(cachePath);
            for (const entry of entries){
                const entryPath = _path.default.join(cachePath, entry);
                const entryStat = _fs.default.statSync(entryPath);
                _logger.logger.verbose(`clearing cache for ${info.name}`);
                if (now.getTime() - entryStat.mtime.getTime() > prunePeriod * MS_IN_A_DAY) {
                    remove(entryPath, entryStat);
                }
            }
        }
    }
}
function getCachePath(info, cacheOptions) {
    const cacheFolder = !cacheOptions.internalCacheFolder ? undefined : `../${cacheOptions.internalCacheFolder}`;
    return _path.default.join(info.packageJsonPath, cacheFolder ?? "../node_modules/.cache/backfill");
}
function remove(entryPath, entryStat) {
    if (entryStat.isDirectory()) {
        _fs.default.rmdirSync(entryPath, {
            recursive: true
        });
    } else {
        _fs.default.unlinkSync(entryPath);
    }
}
