"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
Object.defineProperty(exports, "init", {
    enumerable: true,
    get: ()=>init
});
const _logger = require("../logger");
const _fs = /*#__PURE__*/ _interopRequireDefault(require("fs"));
const _path = /*#__PURE__*/ _interopRequireDefault(require("path"));
const _execa = /*#__PURE__*/ _interopRequireDefault(require("execa"));
function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        default: obj
    };
}
async function init(cwd) {
    _logger.logger.info("initialize lage with a default configuration file");
    let workspaceManager = "yarn";
    try {
        workspaceManager = whichWorkspaceManager(cwd);
    } catch (e) {
        _logger.logger.error("lage requires you to be using a workspace - make sure you are using yarn workspaces, pnpm workspaces, or rush");
    }
    const pipeline = {
        build: [
            "^build"
        ],
        test: [
            "build"
        ],
        lint: []
    };
    const lageConfig = {
        pipeline,
        npmClient: workspaceManager === "yarn" ? "yarn" : "npm"
    };
    const lageConfigFile = _path.default.join(cwd, "lage.config.js");
    if (!_fs.default.existsSync(lageConfigFile)) {
        _fs.default.writeFileSync(lageConfigFile, "module.exports = " + JSON.stringify(lageConfig, null, 2) + ";");
    }
    installLage(cwd, workspaceManager);
    _logger.logger.info(`Lage is initialized! You can now run: ${renderBuildCommand(workspaceManager)}`);
}
function renderBuildCommand(workspaceManager) {
    switch(workspaceManager){
        case "yarn":
            return `yarn lage build`;
        case `pnpm`:
            return `pnpm run lage build`;
        case `rush`:
            return `npm run lage build`;
    }
}
function whichWorkspaceManager(cwd) {
    if (_fs.default.existsSync(_path.default.join(cwd, "rush.json"))) {
        return "rush";
    }
    if (_fs.default.existsSync(_path.default.join(cwd, "yarn.lock"))) {
        return "yarn";
    }
    if (_fs.default.existsSync(_path.default.join(cwd, "pnpm-workspace.yaml"))) {
        return "pnpm";
    }
    throw new Error("not a workspace");
}
async function installLage(cwd, workspaceManager) {
    const packageJson = readPackageJson(cwd);
    const lageVersion = getLageVersion();
    switch(workspaceManager){
        case "yarn":
            packageJson.scripts = packageJson.scripts || {};
            packageJson.devDependencies = packageJson.devDependencies || {};
            packageJson.scripts.lage = "lage";
            packageJson.devDependencies.lage = lageVersion;
            writePackageJson(cwd, packageJson);
            await (0, _execa.default)("yarn", {
                stdio: "inherit"
            });
            break;
        case "pnpm":
            packageJson.scripts = packageJson.scripts || {};
            packageJson.devDependencies = packageJson.devDependencies || {};
            packageJson.scripts.lage = "lage";
            packageJson.devDependencies.lage = lageVersion;
            writePackageJson(cwd, packageJson);
            await (0, _execa.default)("pnpm", [
                "install"
            ], {
                stdio: "inherit"
            });
            break;
        case "rush":
            packageJson.scripts = packageJson.scripts || {};
            packageJson.scripts.lage = `node common/scripts/install-run.js lage@${lageVersion} lage`;
            writePackageJson(cwd, packageJson);
            break;
    }
}
function getLageVersion() {
    const lagePackageJsonFile = require.resolve("../../package.json", {
        paths: [
            __dirname
        ]
    });
    const lagePackageJson = JSON.parse(_fs.default.readFileSync(lagePackageJsonFile, "utf-8"));
    return lagePackageJson.version;
}
function writePackageJson(cwd, packageJson) {
    const packageJsonFile = _path.default.join(cwd, "package.json");
    _fs.default.writeFileSync(packageJsonFile, JSON.stringify(packageJson, null, 2));
}
function readPackageJson(cwd) {
    const packageJsonFile = _path.default.join(cwd, "package.json");
    return JSON.parse(_fs.default.readFileSync(packageJsonFile, "utf-8"));
}
