"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
Object.defineProperty(exports, "NpmLogReporter", {
    enumerable: true,
    get: ()=>NpmLogReporter
});
const _npmlog = /*#__PURE__*/ _interopRequireDefault(require("npmlog"));
const _chalk = /*#__PURE__*/ _interopRequireDefault(require("chalk"));
const _logLevel = require("../LogLevel");
const _formatDuration = require("./formatDuration");
const _taskId = require("../../task/taskId");
const _crypto = /*#__PURE__*/ _interopRequireDefault(require("crypto"));
function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        default: obj
    };
}
const maxLengths = {
    pkg: 0,
    task: 0
};
const colors = {
    info: _chalk.default.white,
    verbose: _chalk.default.gray,
    warn: _chalk.default.white,
    error: _chalk.default.white,
    task: _chalk.default.cyan,
    pkg: _chalk.default.magenta,
    silly: _chalk.default.green
};
const pkgColors = [
    _chalk.default.red,
    _chalk.default.blue,
    _chalk.default.cyanBright,
    _chalk.default.greenBright,
    _chalk.default.yellow,
    _chalk.default.magenta
];
function hashStringToNumber(str) {
    const hash = _crypto.default.createHash("md5");
    hash.update(str);
    const hex = hash.digest("hex").substring(0, 6);
    return parseInt(hex, 16);
}
const pkgNameToIndexInPkgColorArray = new Map();
function getColorForPkg(pkg) {
    if (!pkgNameToIndexInPkgColorArray.has(pkg)) {
        const index = hashStringToNumber(pkg) % pkgColors.length;
        pkgNameToIndexInPkgColorArray.set(pkg, index);
    }
    return pkgColors[pkgNameToIndexInPkgColorArray.get(pkg)];
}
function getTaskLogPrefix(pkg, task) {
    const pkgColor = getColorForPkg(pkg);
    return `${pkgColor(pkg.padStart(maxLengths.pkg))} ${colors.task(task.padStart(maxLengths.task))}`;
}
function normalize(prefixOrMessage, message) {
    if (typeof message === "string") {
        const prefix = prefixOrMessage;
        return {
            prefix,
            message
        };
    } else {
        const prefix1 = "";
        const message1 = prefixOrMessage;
        return {
            prefix: prefix1,
            message: message1
        };
    }
}
function isTaskData(data) {
    return data !== undefined && data.task !== undefined;
}
function isInfoData(data) {
    return data !== undefined && data.command !== undefined;
}
class NpmLogReporter {
    log(entry) {
        if (this.options.logLevel >= entry.level) {
            if (isTaskData(entry.data) && !this.options.grouped) {
                return this.logTaskEntry(entry.data.package, entry.data.task, entry);
            } else if (isTaskData(entry.data) && this.options.grouped) {
                return this.logTaskEntryInGroup(entry.data.package, entry.data.task, entry);
            } else if (isInfoData(entry.data)) {
                return this.logInfoEntry(entry);
            } else {
                return this.logGenericEntry(entry);
            }
        }
    }
    logInfoEntry(entry) {
        const infoData = entry.data;
        const logFn = _npmlog.default[_logLevel.LogLevel[entry.level]];
        const colorFn = colors[_logLevel.LogLevel[entry.level]];
        return logFn("", colorFn(JSON.stringify(infoData, null, 2)));
    }
    logGenericEntry(entry) {
        const normalizedArgs = normalize(entry.msg);
        const logFn = _npmlog.default[_logLevel.LogLevel[entry.level]];
        const colorFn = colors[_logLevel.LogLevel[entry.level]];
        return logFn(normalizedArgs.prefix, colorFn(normalizedArgs.message));
    }
    logTaskEntry(pkg, task, entry) {
        const normalizedArgs = this.options.grouped ? normalize(entry.msg) : normalize(getTaskLogPrefix(pkg, task), entry.msg);
        const logFn = _npmlog.default[_logLevel.LogLevel[entry.level]];
        const colorFn = colors[_logLevel.LogLevel[entry.level]];
        const data = entry.data;
        if (data.status) {
            const pkgColor = getColorForPkg(pkg);
            const pkgTask = this.options.grouped ? `${pkgColor(pkg)} ${_chalk.default.cyan(task)}` : "";
            switch(data.status){
                case "started":
                    return logFn(normalizedArgs.prefix, colorFn(`▶️ start ${pkgTask}`));
                case "completed":
                    return logFn(normalizedArgs.prefix, colorFn(`✔️ done ${pkgTask} - ${(0, _formatDuration.formatDuration)(data.duration)}`));
                case "failed":
                    return logFn(normalizedArgs.prefix, colorFn(`❌ fail ${pkgTask}`));
                case "skipped":
                    return logFn(normalizedArgs.prefix, colorFn(`⏭️ skip ${pkgTask} - ${data.hash}`));
            }
        } else {
            return logFn(normalizedArgs.prefix, colorFn("|  " + normalizedArgs.message));
        }
    }
    logTaskEntryInGroup(pkg, task, logEntry) {
        const taskId = (0, _taskId.getTargetId)(pkg, task);
        this.groupedEntries.set(taskId, this.groupedEntries.get(taskId) || []);
        this.groupedEntries.get(taskId)?.push(logEntry);
        const data = logEntry.data;
        if (data && (data.status === "completed" || data.status === "failed" || data.status === "skipped")) {
            const entries = this.groupedEntries.get(taskId);
            for (const entry of entries){
                this.logTaskEntry(data.package, data.task, entry);
            }
            if (entries.length > 2) {
                this.hr();
            }
        }
    }
    hr() {
        _npmlog.default.info("", "----------------------------------------------");
    }
    summarize(context) {
        const { measures , targets  } = context;
        const { hr  } = this;
        const statusColorFn = {
            completed: _chalk.default.greenBright,
            failed: _chalk.default.redBright,
            skipped: _chalk.default.gray,
            started: _chalk.default.yellow,
            pending: _chalk.default.gray
        };
        _npmlog.default.info("", _chalk.default.cyanBright(`🏗 Summary\n`));
        if (targets.size > 0) {
            for (const wrappedTarget of targets.values()){
                const colorFn = statusColorFn[wrappedTarget.status];
                const target = wrappedTarget.target;
                _npmlog.default.verbose("", getTaskLogPrefix(target.packageName || "[GLOBAL]", target.task), colorFn(`${wrappedTarget.status === "started" ? "started - incomplete" : wrappedTarget.status}${wrappedTarget.duration ? `, took ${(0, _formatDuration.formatDuration)((0, _formatDuration.hrToSeconds)(wrappedTarget.duration))}` : ""}`));
            }
            const successfulTasks = [
                ...targets.values()
            ].filter((t)=>t.status === "completed");
            const skippedTasks = [
                ...targets.values()
            ].filter((t)=>t.status === "skipped");
            _npmlog.default.info("", `[Tasks Count] success: ${successfulTasks.length}, skipped: ${skippedTasks.length}, incomplete: ${targets.size - successfulTasks.length - skippedTasks.length}`);
        } else {
            _npmlog.default.info("", "Nothing has been run.");
        }
        hr();
        if (measures.failedTargets && measures.failedTargets.length > 0) {
            for (const failedTargetId of measures.failedTargets){
                const { packageName , task  } = (0, _taskId.getPackageAndTask)(failedTargetId);
                const taskLogs = targets.get(failedTargetId)?.logger.getLogs();
                _npmlog.default.error("", `[${_chalk.default.magenta(packageName)} ${_chalk.default.cyan(task)}] ${_chalk.default.redBright("ERROR DETECTED")}`);
                if (taskLogs) {
                    for (const entry of taskLogs){
                        // Log each entry separately to prevent truncation
                        _npmlog.default.error("", entry.msg);
                    }
                }
                hr();
            }
        }
        _npmlog.default.info("", `Took a total of ${(0, _formatDuration.formatDuration)((0, _formatDuration.hrToSeconds)(measures.duration))} to complete`);
    }
    constructor(options){
        this.options = options;
        this.groupedEntries = new Map();
        options.logLevel = options.logLevel || _logLevel.LogLevel.info;
        _npmlog.default.level = _logLevel.LogLevel[options.logLevel];
        _npmlog.default.disp = {
            ..._npmlog.default.disp,
            ...options.npmLoggerOptions?.disp
        };
        _npmlog.default.style = {
            ..._npmlog.default.style,
            ...options.npmLoggerOptions?.style
        };
        _npmlog.default.levels = {
            ..._npmlog.default.levels,
            ...options.npmLoggerOptions?.levels
        };
    }
}
